import asyncio
from abc import ABC, abstractmethod
from typing import Any

import colorama
import progressbar

from polyglot import deepl


class Translator(ABC):

    _target_lang: str
    _source_lang: str
    _dispatcher: deepl.Deepl

    def __init__(
        self, target_lang: str, source_lang: str, dispatcher: deepl.Deepl
    ) -> None:
        self._target_lang = target_lang
        self._source_lang = source_lang
        self._dispatcher = dispatcher

    @abstractmethod
    def translate(self, content: Any) -> Any:
        pass


class TextTranslator(Translator):
    def translate(self, content: str) -> str:
        return self._dispatcher.translate(content, self._target_lang, self._source_lang)


class DictionaryTranslator(Translator):

    __progress_bar: progressbar.ProgressBar
    __completion_count: int = 0
    __not_translated_entries: list[str] = []

    def translate(self, content: dict) -> dict:
        self.__set_progress_bar(content)
        self.__translate_dictionary(content)
        self.__print_messages()
        return content

    def __set_progress_bar(self, content: dict) -> None:
        number_of_translations: int = self.__get_number_of_translations(content)
        self.__progress_bar = progressbar.ProgressBar(
            max_value=number_of_translations, redirect_stdout=True
        )

    def __get_number_of_translations(self, dictionary: dict) -> int:
        return sum(
            self.__get_number_of_translations(value) if isinstance(value, dict) else 1
            for key, value in dictionary.items()
        )

    def __translate_dictionary(self, dictionary: dict) -> None:

        for key, value in dictionary.items():

            if isinstance(value, dict):
                self.__translate_dictionary(value)

            else:
                dictionary[key] = self.__translate_entry(value)
                self.__completion_count += 1
                self.__progress_bar.update(self.__completion_count)

    def __translate_entry(self, entry: str) -> str:
        translation: str = self._dispatcher.translate(
            entry, self._target_lang, self._source_lang
        )
        if not translation:
            self.__not_translated_entries.append(entry)
        return translation if translation else entry

    def __print_messages(self) -> None:
        print("\nTranslation completed.")
        if len(self.__not_translated_entries) > 0:
            print(
                f"{colorama.Fore.YELLOW}\nThe following entries have not been translated:\n"
            )
            for entry in self.__not_translated_entries:
                print(f'{colorama.Fore.RESET}"{entry}"\n')


class DocumentTranslator(Translator):

    __document_id: str
    __document_key: str
    __translated_file: bytes

    def translate(self, content: str) -> bytes:
        document_data: dict[str, str] = self._dispatcher.translate_document(
            content, self._target_lang, self._source_lang
        )
        self.__document_id = document_data["document_id"]
        self.__document_key = document_data["document_key"]
        asyncio.run(self.__download_document_when_ready())
        return self.__translated_file

    async def __download_document_when_ready(self) -> None:
        status_data = self._dispatcher.check_document_status(
            self.__document_id, self.__document_key
        )
        status: str = status_data["status"]

        if status == "done":
            billed_characters: str = status_data["billed_characters"]
            print(f"Translation completed. Billed characters: {billed_characters}.")
            self.__translated_file = self.__download_target_file()
            return

        # * sometimes there are no seconds even if it's still translating
        if "seconds_remaining" in status_data:
            print(f'Remaining {status_data["seconds_remaining"]} seconds...')

        await self.__download_document_when_ready()

    def __download_target_file(self) -> bytes:
        return self._dispatcher.download_translated_document(
            self.__document_id, self.__document_key
        )



import colorama


def get_truncated_text(text: str, limit: int) -> str:
    return text[:limit] + "..." if len(text) > limit else text


def get_color_by_percentage(percentage: int) -> str:
    if percentage > 90:
        return colorama.Fore.RED
    if percentage > 60:
        return colorama.Fore.YELLOW
    return colorama.Fore.RESET



import colorama


class DeeplError(Exception):
    status_code: int
    message: str

    def __init__(self, status_code: int, message: str):
        self.status_code = status_code
        self.message = message
        if status_code == 403:
            message = f"{message}.\nCheck that your licence is valid"
        super().__init__(
            f"\n\n{colorama.Fore.RED}Status code: {status_code}.\nMessage: {message}.\n"
        )


class HandlerError(Exception):
    source_file: str
    message: str

    def __init__(self, source_file: str, message: str):
        self.source_file = source_file
        self.message = message
        super().__init__(
            f"\n\n{colorama.Fore.RED}Cannot read {self.source_file}.\nMessage: {message}.\n"
        )



import os
from dataclasses import dataclass
from typing import Any
import colorama
from colorama import init
from polyglot import deepl, handlers, arguments, license, translators

# ! Do not move colorama init. Autoreset works only here
init(autoreset=True)


DOCUMENTS_SUPPORTED_BY_DEEPL: list[str] = [".docx", ".pptx", ".html", ".htm"]


@dataclass
class FileTranslator:
    handler: handlers.FileHandler
    translator: translators.Translator


class Polyglot:
    __arguments: arguments.Arguments
    __dispatcher: deepl.Deepl

    def __init__(self, arguments: arguments.Arguments):
        self.__arguments = arguments

    @property
    def __license_manager(self) -> license.LicenseManager:
        return self.__arguments.license_manager

    def execute_command(self):

        if self.__arguments.action == "set_license":
            self.__license_manager.set_license()
            return

        self.__dispatcher = deepl.Deepl(
            license_manager=self.__license_manager,
        )

        if self.__arguments.action == "translate":
            file_translator: FileTranslator = self.__get_file_translator()
            content: Any = file_translator.handler.read()
            translated_content: Any = file_translator.translator.translate(content)
            file_translator.handler.write(translated_content)
            print(f"\n{colorama.Fore.GREEN}Finish.\n{colorama.Fore.RESET}")

        elif self.__arguments.action == "print_supported_languages":
            self.__dispatcher.print_supported_languages()

        elif self.__arguments.action == "print_usage_info":
            self.__dispatcher.print_usage_info()

    def __get_file_translator(self) -> FileTranslator:
        extension: str = os.path.splitext(self.__arguments.source_file)[1]
        handler: handlers.FileHandler = self.__get_handler(extension)
        translator: translators.Translator = self.__get_translator(extension)
        return FileTranslator(handler=handler, translator=translator)

    def __get_handler(self, extension: str) -> handlers.FileHandler:

        file_handler_options: dict[str, str] = {
            "source_file": self.__arguments.source_file,
            "output_directory": self.__arguments.output_directory,
            "target_lang": self.__arguments.target_lang,
        }

        if extension in DOCUMENTS_SUPPORTED_BY_DEEPL:
            return handlers.DocumentHandler(**file_handler_options)

        if extension == ".json":
            return handlers.JSONHandler(**file_handler_options)

        if extension == ".po":
            return handlers.POHandler(**file_handler_options)

        return handlers.TextHandler(**file_handler_options)

    def __get_translator(self, extension: str) -> translators.Translator:

        translator_options: dict[str, Any] = {
            "target_lang": self.__arguments.target_lang,
            "source_lang": self.__arguments.source_lang,
            "dispatcher": self.__dispatcher,
        }

        if extension in DOCUMENTS_SUPPORTED_BY_DEEPL:
            return translators.DocumentTranslator(**translator_options)

        if extension == ".json" or extension == ".po":
            return translators.DictionaryTranslator(**translator_options)

        return translators.TextTranslator(**translator_options)
